from pydoautomator.droplet import Droplet
from pydoautomator.adapters import ApiAdapter
import asyncio
from pydoautomator.errors import DropletCreationError, FloatingIpAssignmentError, TurnoffDropletError, DestroyDropletError


class Automator:
    """Here are the main methods / functions
    Call this class passing the Digital Ocean TOKEN.

    Raises:
        TurnoffDropletError: Error while shutting down droplet
        FloatingIpAssignmentError: Error while assigning floating ip
        DestroyDropletError: Error while destroying droplet
        DropletCreationError: Error while creating droplet

    """

    do_token = ''

    requests = ''
    __base_url = 'https://api.digitalocean.com/v2'

    def __init__(self, do_token: str):
        """constructor

        Args:
            do_token (str): digital ocean token
        """

        self.do_token = do_token
        self.api_adapter = ApiAdapter(self.do_token)
        self.requests = self.api_adapter.requests
        self.__base_url = 'https://api.digitalocean.com/v2'

    def turnoff_droplet(self, droplet_id: int) -> str:
        """Turnoff / Shutdown Droplet

        Args:
            droplet_id (int): droplet id

        Raises:
            TurnoffDropletError: Error if droplet is not shutdown

        Returns:
            str: `completed`
        """

        headers = {'Content-Type': 'application/json'}
        data = {'type': 'shutdown'}

        response = self.requests.post(
            self.__base_url+'/droplets/'+str(droplet_id)+'/actions',
            headers=headers,
            json=data
        )

        if response.status_code != 201:
            raise TurnoffDropletError(response.json())

        action_id = response.json()['action']['id']

        loop2 = asyncio.get_event_loop()
        loop2.run_until_complete(
            self.__wait_till_action_complete(action_id)
        )
        return "completed"

    def get_all_droplets(self) -> list:
        """Get all droplets from Digital Ocean

        Returns:
            list: list of droplets (dict)
        """

        response = self.requests.get(self.__base_url+'/droplets')
        droplets = response.json()['droplets']

        while 'next' in response.json()['links']['pages']:
            response = self.requests.get(
                response.json()['links']['pages']['next'])
            droplets = droplets + response.json()['droplets']

        return droplets

    def assign_floating_ip_to_droplet(self, floating_ip: str, droplet_id: int) -> str:
        """Assigns a floating ip to a droplet

        Args:
            floating_ip (str): Floating IP (i.e.: `125.68.75.2`)
            droplet_id (int): The droplet ID

        Raises:
            FloatingIpAssignmentError: if not completed

        Returns:
            str: return `completed` if completed
        """

        url = self.__base_url + '/floating_ips/' + floating_ip + '/actions'
        data = {
            "type": "assign",
            "droplet_id": droplet_id
        }

        headers = {'Content-Type': 'application/json'}
        response = self.requests.post(url, json=data, headers=headers)

        if response.status_code != 201:
            raise FloatingIpAssignmentError(response.json())

        action_id = response.json()['action']['id']

        loop2 = asyncio.get_event_loop()
        loop2.run_until_complete(
            self.__wait_till_action_complete(action_id)
        )
        if response.status_code == 201:
            return 'completed'

    def create_droplet_from_snapshot(self, droplet: Droplet) -> int:
        """Creates droplet from snapshot/image

        Args:
            droplet (Droplet): Droplet object

        Returns:
            int: `droplet_id` from created droplet
        """

        headers = {'Content-Type': 'application/json'}
        created_droplet = self.requests.post(
            self.__base_url + '/droplets', data=droplet.json(), headers=headers)

        action_id = created_droplet.json()['links']['actions'][0]['id']
        loop = asyncio.get_event_loop()

        loop.run_until_complete(
            self.__wait_till_action_complete(action_id)
        )

        return created_droplet.json()['droplet']['id']

    def destroy_droplet(self, droplet_id: int) -> str:
        """Destroy / Delete droplet

        Args:
            droplet_id (int): droplet id to be destroyed

        Raises:
            DestroyDropletError: if droplet is not destroyed

        Returns:
            str: `completed` if destroyed
        """

        response = self.requests.delete(
            self.__base_url + '/droplets/' + str(droplet_id)
        )

        if response.status_code != 204:
            raise DestroyDropletError(response.json())

        return 'completed'

    async def __wait_till_action_complete(self, action_id: int) -> str:
        action_status = self.__check_action_status(action_id)

        while action_status == 'in-progress':
            await asyncio.sleep(5)
            action_status = self.__check_action_status(action_id)

        if action_status == 'errored':
            raise DropletCreationError

        return action_status

    def __check_action_status(self, action_id: int) -> str:
        """Checks digital ocean action status

        Args:
            action_id (int): Digital Ocean action id

        Returns:
            str: action status (`in-progress`, `completed`, `errored`)
        """
        print('Entered __check_action_status')

        response = self.requests.get(
            self.__base_url + '/actions/' + str(action_id)
        )
        print(response.json())

        return response.json()['action']['status']



from .droplet import Droplet
from .automator import Automator

# creates version from metadata
try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    import importlib_metadata

__version__ = importlib_metadata.version(__name__)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pydoautomator']

package_data = \
{'': ['*']}

install_requires = \
['importlib-metadata>=2.0.0,<3.0.0',
 'pydantic>=1.6.1,<2.0.0',
 'requests>=2.24.0,<3.0.0']

setup_kwargs = {
    'name': 'pydoautomator',
    'version': '0.8.3',
    'description': 'The Python lib for digital ocean automation',
    'long_description': '# pydoautomator\n\n[![codecov](https://codecov.io/gh/christian-hawk/pydoautomator/branch/master/graph/badge.svg)](https://codecov.io/gh/christian-hawk/pydoautomator)  [![DeepSource](https://deepsource.io/gh/christian-hawk/pydoautomator.svg/?label=active+issues&show_trend=true)](https://deepsource.io/gh/christian-hawk/pydoautomator/?ref=repository-badge) [![Actions Status](https://github.com/christian-hawk/pydoautomator/workflows/Python/badge.svg)](https://github.com/christian-hawk/pydoautomator/actions)\n\nThe Digital Ocean python automation lib\n\n## Install\n\n`pip install pydoautomator`\n\n## Simple as that\n\n``` python\nfrom pydoautomator import Automator, Droplet\ndigital_ocean_token = \'my-digital-ocean-api-token\'\n\naut = Automator(digital_ocean_token)\n\ndroplet_data = {\n    "name" : "t1.techno24x7.com",\n    "region" : "nyc1",\n    "size" : "s-8vcpu-16gb",\n    "image" : 68259296, # snapshot id\n    "ssh_keys" : [27410347, 27608055, 27590881],\n    "private_networking" : True,\n    "vpc_uuid" : "47e5c00a-2b23-4dac-bed4-0e44659941f3",\n    "monitoring" : True\n    "tags" : ["tests"]\n}\n\ndroplet = Droplet(**droplet_data)\n\naut.create_droplet_from_snapshot(droplet)\n\n```\n\n### Assign floating ip to droplet\n\n```python\ndroplet_id = 152412424\nfloating_ip = \'164.90.252.72\'\naction_status = aut.assign_floating_ip_to_droplet(floating_ip, droplet_id)\n\nif action_status == \'completed\':\n    print(\'floating_ip assigned to droplet!\')\n```\n\n### Create droplet and assign floating ip as soon as droplet created\n\n```python\ndigital_ocean_token = \'my-super-cool-digital-ocean-api-token\'\n\naut = Automator(digital_ocean_token)\n\ndroplet_data = {\n    "name": "t1.techno24x7.com",\n    "region": "nyc1",\n    "size": "s-8vcpu-16gb",\n    "image": 70649304, # snapshot id\n    "ssh_keys": [27410347, 27608055, 27590881],\n    "private_networking": True,\n    "vpc_uuid": "47e5c00a-2b23-4dac-bed4-0e44659941f3",\n    "monitoring": True\n}\n\ndroplet = Droplet(**droplet_data)\n\ndroplet_id = aut.create_droplet_from_snapshot(droplet)\n\n\nfloating_ip = \'164.90.252.72\'\naction_status = aut.assign_floating_ip_to_droplet(floating_ip, droplet_id)\n\nif action_status == \'completed\':\n    print(\'floating_ip assigned to droplet!\')\n```\n\n### Shutdown / turnoff droplet\n\n```python\ndroplet_id = 123456\naut.turnoff_droplet(droplet_id)\n```\n\n### Destroy droplet by id\n\n```python\ndroplet_id = 123456\naut.destroy_droplet(droplet_id)\n```\n\n### Get all droplets\n\n```python\ndroplets_list = aut.get_all_droplets(droplet_id)\n```\n',
    'author': 'Christian Eland',
    'author_email': 'eland.christian@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/christian-hawk/pydoautomator',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
